@@ -563,6 +563,11 @@ class Controller(object):
             if 'validate' not in attr_vals:
                 continue
             for rule in attr_vals['validate']:
+                # skip validating vnfd_id when vnfd_template is specified to
+                # create vnf
+                if resource == 'vnf' and 'vnfd_template' in body['vnf'] and \
+                   attr == "vnfd_id" and is_create:
+                    continue
                 res = attributes.validators[rule](res_dict[attr],
                                                   attr_vals['validate'][rule])
                 if res:
