@@ -58,27 +58,6 @@ core_opts = [
                       "means no limit")),
     cfg.StrOpt('host', default=utils.get_hostname(),
                help=_("The hostname Tacker is running on")),
-    cfg.StrOpt('nova_url',
-               default='http://127.0.0.1:8774/v2',
-               help=_('URL for connection to nova')),
-    cfg.StrOpt('nova_admin_username',
-               help=_('Username for connecting to nova in admin context')),
-    cfg.StrOpt('nova_admin_password',
-               help=_('Password for connection to nova in admin context'),
-               secret=True),
-    cfg.StrOpt('nova_admin_tenant_id',
-               help=_('The uuid of the admin nova tenant')),
-    cfg.StrOpt('nova_admin_auth_url',
-               default='http://localhost:5000/v2.0',
-               help=_('Authorization URL for connecting to nova in admin '
-                      'context')),
-    cfg.StrOpt('nova_ca_certificates_file',
-               help=_('CA file for novaclient to verify server certificates')),
-    cfg.BoolOpt('nova_api_insecure', default=False,
-                help=_("If True, ignore any SSL validation issues")),
-    cfg.StrOpt('nova_region_name',
-               help=_('Name of nova region to use. Useful if keystone manages'
-                      ' more than one region.')),
 ]
 
 core_cli_opts = [
