@@ -249,6 +249,10 @@ class AlarmUrlInvalid(BadRequest):
     message = _("Invalid alarm url for VNF %(vnf_id)s")
 
 
+class TriggerNotFound(NotFound):
+    message = _("Trigger %(trigger_name)s does not exist for VNF %(vnf_id)s")
+
+
 class VnfPolicyNotFound(NotFound):
     message = _("Policy %(policy)s does not exist for VNF %(vnf_id)s")
 
@@ -265,3 +269,7 @@ class VnfPolicyTypeInvalid(BadRequest):
 
 class DuplicateResourceName(TackerException):
     message = _("%(resource)s with name %(name)s already exists")
+
+
+class InvalidParam(TackerException):
+    message = _("Param values must be a dict type")
