@@ -60,7 +60,7 @@ class Vim(model_base.BASE,
 class VimAuth(model_base.BASE, models_v1.HasId):
     vim_id = sa.Column(types.Uuid, sa.ForeignKey('vims.id'),
                        nullable=False)
-    password = sa.Column(sa.String(128), nullable=False)
+    password = sa.Column(sa.String(255), nullable=False)
     auth_url = sa.Column(sa.String(255), nullable=False)
     vim_project = sa.Column(types.Json, nullable=False)
     auth_cred = sa.Column(types.Json, nullable=False)
@@ -219,13 +219,6 @@ class NfvoPluginDb(nfvo.NFVOPluginBase, db_base.CommonDbMixin):
                            'updated_at': timeutils.utcnow()})
         return self._make_vim_dict(vim_db)
 
-    # Deprecated. Will be removed in Ocata release
-    def get_vim_by_name(self, context, vim_name, fields=None,
-                        mask_password=True):
-        vim_db = self._get_by_name(context, Vim, vim_name)
-        return self._make_vim_dict(vim_db, mask_password=mask_password
-                                   )if vim_db else None
-
     def _validate_default_vim(self, context, vim, vim_id=None):
         if not vim.get('is_default'):
             return True
