@@ -31,14 +31,6 @@ from tacker.services import service_base
 LOG = logging.getLogger(__name__)
 
 
-class InfraDriverNotSpecified(exceptions.InvalidInput):
-    message = _('infra driver is not specified')
-
-
-class MGMTDriverNotSpecified(exceptions.InvalidInput):
-    message = _('mgmt driver is not specified')
-
-
 class MultipleMGMTDriversSpecified(exceptions.InvalidInput):
     message = _('More than one MGMT Driver per vnfd is not supported')
 
@@ -103,14 +95,6 @@ class HeatTranslatorFailed(exceptions.InvalidInput):
     message = _("heat-translator failed: - %(error_msg_details)s")
 
 
-class InputValuesMissing(exceptions.InvalidInput):
-    message = _("Parameter input values missing for the key '%(key)s'")
-
-
-class ParamYAMLInputMissing(exceptions.InvalidInput):
-    message = _("Parameter YAML input missing")
-
-
 class HeatClientException(exceptions.TackerException):
     message = _("%(msg)s")
 
@@ -153,6 +137,25 @@ class VNFInactive(exceptions.InvalidInput):
     message = _("VNF %(vnf_id)s is not in Active state %(message)s")
 
 
+class MetadataNotMatched(exceptions.InvalidInput):
+    message = _("Metadata for alarm policy is not matched")
+
+
+class InvalidSubstitutionMapping(exceptions.InvalidInput):
+    message = _("Input for substitution mapping requirements are not"
+                " valid for %(requirement)s. They must be in the form"
+                " of list with two entries")
+
+
+class SMRequirementMissing(exceptions.InvalidInput):
+    message = _("All the requirements for substitution_mappings are not"
+                " provided. Missing requirement for %(requirement)s")
+
+
+class InvalidParamsForSM(exceptions.InvalidInput):
+    message = _("Please provide parameters for substitution mappings")
+
+
 def _validate_service_type_list(data, valid_values=None):
     if not isinstance(data, list):
         msg = _("invalid data format for service list: '%s'") % data
@@ -215,20 +218,6 @@ RESOURCE_ATTRIBUTE_MAP = {
             'is_visible': True,
             'default': attr.ATTR_NOT_SPECIFIED,
         },
-        'infra_driver': {
-            'allow_post': True,
-            'allow_put': False,
-            'validate': {'type:string': None},
-            'is_visible': True,
-            'default': "",
-        },
-        'mgmt_driver': {
-            'allow_post': True,
-            'allow_put': False,
-            'validate': {'type:string': None},
-            'is_visible': True,
-            'default': "",
-        },
         'attributes': {
             'allow_post': True,
             'allow_put': False,
@@ -247,6 +236,12 @@ RESOURCE_ATTRIBUTE_MAP = {
             'allow_put': False,
             'is_visible': True,
         },
+        'template_source': {
+            'allow_post': False,
+            'allow_put': False,
+            'is_visible': True,
+            'default': 'onboarded'
+        },
     },
 
     'vnfs': {
@@ -269,6 +264,7 @@ RESOURCE_ATTRIBUTE_MAP = {
             'allow_put': False,
             'validate': {'type:uuid': None},
             'is_visible': True,
+            'default': None
         },
         'vim_id': {
             'allow_post': True,
@@ -336,6 +332,13 @@ RESOURCE_ATTRIBUTE_MAP = {
             'allow_put': False,
             'is_visible': True,
         },
+        'vnfd_template': {
+            'allow_post': True,
+            'allow_put': False,
+            'validate': {'type:dict_or_none': None},
+            'is_visible': True,
+            'default': None,
+        },
     },
 }
 
