@@ -124,8 +124,19 @@ class TestVNFMPlugin(db_base.SqlTestCase):
             tenant_id='ad7ebc56538745a08ef7c5e97f8bd437',
             name='fake_template',
             description='fake_template_description',
-            infra_driver='fake_driver',
-            mgmt_driver='fake_mgmt_driver')
+            template_source='onboarded')
+        session.add(device_template)
+        session.flush()
+        return device_template
+
+    def _insert_dummy_device_template_inline(self):
+        session = self.context.session
+        device_template = vnfm_db.VNFD(
+            id='d58bcc4e-d0cf-11e6-bf26-cec0c932ce01',
+            tenant_id='ad7ebc56538745a08ef7c5e97f8bd437',
+            name='tmpl-koeak4tqgoqo8cr4-dummy_inline_vnf',
+            description='inline_fake_template_description',
+            template_source='inline')
         session.add(device_template)
         session.flush()
         return device_template
@@ -197,7 +208,7 @@ class TestVNFMPlugin(db_base.SqlTestCase):
             auth_url='http://localhost:5000',
             vim_project={'name': 'test_project'},
             auth_cred={'username': 'test_user', 'user_domain_id': 'default',
-                       'project_domain_d': 'default'})
+                       'project_domain_id': 'default'})
         session.add(vim_db)
         session.add(vim_auth_db)
         session.flush()
@@ -221,6 +232,7 @@ class TestVNFMPlugin(db_base.SqlTestCase):
         self.assertIn('attributes', result)
         self.assertIn('created_at', result)
         self.assertIn('updated_at', result)
+        self.assertIn('template_source', result)
         yaml_dict = yaml.safe_load(utils.tosca_vnfd_openwrt)
         mock_tosca_template.assert_called_once_with(
             a_file=False, yaml_dict_tpl=yaml_dict)
@@ -228,7 +240,7 @@ class TestVNFMPlugin(db_base.SqlTestCase):
         mock_update_imports.assert_called_once_with(yaml_dict)
         self._cos_db_plugin.create_event.assert_called_once_with(
             self.context, evt_type=constants.RES_EVT_CREATE, res_id=mock.ANY,
-            res_state=constants.RES_EVT_VNFD_NA_STATE,
+            res_state=constants.RES_EVT_ONBOARDED,
             res_type=constants.RES_TYPE_VNFD, tstamp=mock.ANY)
 
     def test_create_vnfd_no_service_types(self):
@@ -246,7 +258,7 @@ class TestVNFMPlugin(db_base.SqlTestCase):
                           self.vnfm_plugin.create_vnfd,
                           self.context, vnfd_obj)
 
-    def test_create_vnf(self):
+    def test_create_vnf_with_vnfd(self):
         self._insert_dummy_device_template()
         vnf_obj = utils.get_dummy_vnf_obj()
         result = self.vnfm_plugin.create_vnf(self.context, vnf_obj)
@@ -270,6 +282,35 @@ class TestVNFMPlugin(db_base.SqlTestCase):
             res_state=mock.ANY, res_type=constants.RES_TYPE_VNF,
             tstamp=mock.ANY, details=mock.ANY)
 
+    @mock.patch('tacker.vnfm.plugin.VNFMPlugin.create_vnfd')
+    def test_create_vnf_from_template(self, mock_create_vnfd):
+        self._insert_dummy_device_template_inline()
+        mock_create_vnfd.return_value = {'id':
+                'd58bcc4e-d0cf-11e6-bf26-cec0c932ce01'}
+        vnf_obj = utils.get_dummy_inline_vnf_obj()
+        result = self.vnfm_plugin.create_vnf(self.context, vnf_obj)
+        self.assertIsNotNone(result)
+        self.assertIn('id', result)
+        self.assertIn('instance_id', result)
+        self.assertIn('status', result)
+        self.assertIn('attributes', result)
+        self.assertIn('mgmt_url', result)
+        self.assertIn('created_at', result)
+        self.assertIn('updated_at', result)
+        mock_create_vnfd.assert_called_once_with(mock.ANY, mock.ANY)
+        self._device_manager.invoke.assert_called_with('test_vim',
+                                                       'create',
+                                                       plugin=mock.ANY,
+                                                       context=mock.ANY,
+                                                       vnf=mock.ANY,
+                                                       auth_attr=mock.ANY)
+        self._pool.spawn_n.assert_called_once_with(mock.ANY)
+        self._cos_db_plugin.create_event.assert_called_with(
+            self.context, evt_type=constants.RES_EVT_CREATE,
+            res_id=mock.ANY,
+            res_state=mock.ANY, res_type=constants.RES_TYPE_VNF,
+            tstamp=mock.ANY, details=mock.ANY)
+
     def test_show_vnf_details_vnf_inactive(self):
         self._insert_dummy_device_template()
         vnf_obj = utils.get_dummy_vnf_obj()
@@ -386,25 +427,55 @@ class TestVNFMPlugin(db_base.SqlTestCase):
     def test_scale_vnf_in(self):
         self._test_scale_vnf('in', constants.PENDING_SCALE_IN)
 
-    @patch('tacker.vnfm.monitor.ActionPolicy')
-    def test_create_vnf_trigger_respawn(self, mock_action_policy):
-        action_policy_cls = mock_action_policy.return_value
-        action_policy_cls.get_policy.return_value = mock.Mock()
+    def _get_dummy_active_vnf(self, vnfd_template):
+        dummy_vnf = utils.get_dummy_device_obj()
+        dummy_vnf['vnfd']['attributes']['vnfd'] = vnfd_template
+        dummy_vnf['status'] = 'ACTIVE'
+        dummy_vnf['instance_id'] = '4c00108e-c69d-4624-842d-389c77311c1d'
+        dummy_vnf['vim_id'] = '437ac8ef-a8fb-4b6e-8d8a-a5e86a376e8b'
+        return dummy_vnf
+
+    def _test_create_vnf_trigger(self, policy_name, action_value):
         vnf_id = "6261579e-d6f3-49ad-8bc3-a9cb974778fe"
-        trigger_request = {"trigger": {"action_name": "respawn", "params": {
+        trigger_request = {"trigger": {"action_name": action_value, "params": {
             "credential": "026kll6n", "data": {"current": "alarm",
                                                'alarm_id':
                                     "b7fa9ffd-0a4f-4165-954b-5a8d0672a35f"}},
-            "policy_name": "vdu1_cpu_usage_monitoring_policy"}}
-        self._insert_dummy_device_template()
-        self._insert_dummy_vnfd_attributes(utils.vnfd_alarm_tosca_template)
-        self._insert_dummy_device()
-        expected_result = {"action_name": "respawn", "params": {
+            "policy_name": policy_name}}
+        expected_result = {"action_name": action_value, "params": {
             "credential": "026kll6n", "data": {"current": "alarm",
             "alarm_id": "b7fa9ffd-0a4f-4165-954b-5a8d0672a35f"}},
-            "policy_name": "vdu1_cpu_usage_monitoring_policy"}
+            "policy_name": policy_name}
         self._vnf_alarm_monitor.process_alarm_for_vnf.return_value = True
-
         trigger_result = self.vnfm_plugin.create_vnf_trigger(
             self.context, vnf_id, trigger_request)
         self.assertEqual(expected_result, trigger_result)
+
+    @patch('tacker.db.vnfm.vnfm_db.VNFMPluginDb.get_vnf')
+    @patch('tacker.vnfm.monitor.ActionPolicy.get_policy')
+    def test_create_vnf_trigger_respawn(self, mock_get_policy, mock_get_vnf):
+        dummy_vnf = self._get_dummy_active_vnf(
+            utils.vnfd_alarm_respawn_tosca_template)
+        mock_get_vnf.return_value = dummy_vnf
+        mock_action_class = mock.Mock()
+        mock_get_policy.return_value = mock_action_class
+        self._test_create_vnf_trigger(policy_name="vdu_hcpu_usage_respawning",
+                                      action_value="respawn")
+        mock_get_policy.assert_called_once_with('respawn', 'test_vim')
+        mock_action_class.execute_action.assert_called_once_with(
+            self.vnfm_plugin, dummy_vnf)
+
+    @patch('tacker.db.vnfm.vnfm_db.VNFMPluginDb.get_vnf')
+    @patch('tacker.vnfm.monitor.ActionPolicy.get_policy')
+    def test_create_vnf_trigger_scale(self, mock_get_policy, mock_get_vnf):
+        dummy_vnf = self._get_dummy_active_vnf(
+            utils.vnfd_alarm_scale_tosca_template)
+        mock_get_vnf.return_value = dummy_vnf
+        mock_action_class = mock.Mock()
+        mock_get_policy.return_value = mock_action_class
+        scale_body = {'scale': {'policy': 'SP1', 'type': 'out'}}
+        self._test_create_vnf_trigger(policy_name="vdu_hcpu_usage_scaling_out",
+                                      action_value="SP1-out")
+        mock_get_policy.assert_called_once_with('scaling', 'test_vim')
+        mock_action_class.execute_action.assert_called_once_with(
+            self.vnfm_plugin, dummy_vnf, scale_body)
