@@ -21,7 +21,6 @@ import yaml
 import eventlet
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_log import versionutils
 from oslo_utils import excutils
 from toscaparser.tosca_template import ToscaTemplate
 
@@ -114,7 +113,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
     """
     OPTS = [
         cfg.ListOpt(
-            'infra_driver', default=['nova', 'heat', 'noop', 'openstack'],
+            'infra_driver', default=['noop', 'openstack'],
             help=_('Hosting vnf drivers tacker plugin will use')),
     ]
     cfg.CONF.register_opts(OPTS, 'tacker')
@@ -146,18 +145,11 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
         elif isinstance(template, str):
             self._report_deprecated_yaml_str()
         if "tosca_definitions_version" not in template:
-            versionutils.report_deprecated_feature(LOG, 'VNFD legacy vnfds'
-                ' are deprecated since Mitaka release and will be removed in'
-                ' Ocata release. Please use NFV TOSCA vnfds.')
+            raise exceptions.Invalid('Not a valid template: '
+                                     'tosca_definitions_version is missing.')
 
         LOG.debug(_('vnfd %s'), vnfd_data)
 
-        if 'infra_driver' in vnfd_data or 'mgmt_driver' in vnfd_data:
-            versionutils.report_deprecated_feature(LOG, "Deriving "
-                "infra_driver and mgmt_driver from VNFD API is deprecated and"
-                " will be removed in Ocata. infra_driver will be automatically"
-                " derived from target vim type. mgmt_driver will be derived "
-                "from TOSCA template values.")
         name = vnfd_data['name']
         if self._get_by_name(context, vnfm_db.VNFD, name):
             raise exceptions.DuplicateResourceName(resource='VNFD', name=name)
@@ -171,6 +163,11 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
             # framework doesn't know what services are valid for now.
             # so doesn't check it here yet.
             pass
+        if 'template_source' in vnfd_data:
+            template_source = vnfd_data.get('template_source')
+        else:
+            template_source = 'onboarded'
+        vnfd['vnfd']['template_source'] = template_source
 
         self._parse_template_input(vnfd)
         return super(VNFMPlugin, self).create_vnfd(
@@ -185,53 +182,29 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
         inner_vnfd_dict = yaml.load(vnfd_yaml)
         LOG.debug(_('vnfd_dict: %s'), inner_vnfd_dict)
 
-        if 'tosca_definitions_version' in inner_vnfd_dict:
-            # Prepend the tacker_defs.yaml import file with the full
-            # path to the file
-            toscautils.updateimports(inner_vnfd_dict)
+        # Prepend the tacker_defs.yaml import file with the full
+        # path to the file
+        toscautils.updateimports(inner_vnfd_dict)
 
-            try:
-                tosca = ToscaTemplate(a_file=False,
-                                      yaml_dict_tpl=inner_vnfd_dict)
-            except Exception as e:
-                LOG.exception(_("tosca-parser error: %s"), str(e))
-                raise vnfm.ToscaParserFailed(error_msg_details=str(e))
+        try:
+            tosca = ToscaTemplate(a_file=False,
+                                  yaml_dict_tpl=inner_vnfd_dict)
+        except Exception as e:
+            LOG.exception(_("tosca-parser error: %s"), str(e))
+            raise vnfm.ToscaParserFailed(error_msg_details=str(e))
 
-            if ('description' not in vnfd_dict or
-                    vnfd_dict['description'] == ''):
-                vnfd_dict['description'] = inner_vnfd_dict.get(
-                    'description', '')
-            if (('name' not in vnfd_dict or
-                    not len(vnfd_dict['name'])) and
-                    'metadata' in inner_vnfd_dict):
-                vnfd_dict['name'] = inner_vnfd_dict['metadata'].get(
-                    'template_name', '')
+        if ('description' not in vnfd_dict or
+                vnfd_dict['description'] == ''):
+            vnfd_dict['description'] = inner_vnfd_dict.get(
+                'description', '')
+        if (('name' not in vnfd_dict or
+                not len(vnfd_dict['name'])) and
+                'metadata' in inner_vnfd_dict):
+            vnfd_dict['name'] = inner_vnfd_dict['metadata'].get(
+                'template_name', '')
 
-            vnfd_dict['mgmt_driver'] = toscautils.get_mgmt_driver(
-                tosca)
-        else:
-            KEY_LIST = (('name', 'template_name'),
-                        ('description', 'description'))
-
-            vnfd_dict.update(
-                dict((key, inner_vnfd_dict[vnfd_key]) for (key, vnfd_key)
-                     in KEY_LIST
-                     if ((key not in vnfd_dict or
-                          vnfd_dict[key] == '') and
-                         vnfd_key in inner_vnfd_dict and
-                         inner_vnfd_dict[vnfd_key] != '')))
-
-            service_types = inner_vnfd_dict.get(
-                'service_properties', {}).get('type', [])
-            if service_types:
-                vnfd_dict.setdefault('service_types', []).extend(
-                    [{'service_type': service_type}
-                    for service_type in service_types])
-            # TODO(anyone)  - this code assumes one mgmt_driver per VNFD???
-            for vdu in inner_vnfd_dict.get('vdus', {}).values():
-                mgmt_driver = vdu.get('mgmt_driver')
-                if mgmt_driver:
-                    vnfd_dict['mgmt_driver'] = mgmt_driver
+        vnfd_dict['mgmt_driver'] = toscautils.get_mgmt_driver(
+            tosca)
         LOG.debug(_('vnfd %s'), vnfd)
 
     def add_vnf_to_monitor(self, vnf_dict, infra_driver):
@@ -249,17 +222,19 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
             LOG.debug('hosting_vnf: %s', hosting_vnf)
             self._vnf_monitor.add_hosting_vnf(hosting_vnf)
 
-    def add_alarm_url_to_vnf(self, vnf_dict):
+    def add_alarm_url_to_vnf(self, context, vnf_dict):
         vnfd_yaml = vnf_dict['vnfd']['attributes'].get('vnfd', '')
         vnfd_dict = yaml.load(vnfd_yaml)
         if vnfd_dict and vnfd_dict.get('tosca_definitions_version'):
             polices = vnfd_dict['topology_template'].get('policies', [])
             for policy_dict in polices:
-                name, policy = policy_dict.items()[0]
+                name, policy = list(policy_dict.items())[0]
                 if policy['type'] in constants.POLICY_ALARMING:
-                    alarm_url = self._vnf_alarm_monitor.update_vnf_with_alarm(
-                        vnf_dict, name, policy)
-                    vnf_dict['attributes']['alarm_url'] = alarm_url
+                    alarm_url =\
+                        self._vnf_alarm_monitor.update_vnf_with_alarm(
+                            self, context, vnf_dict, policy)
+                    vnf_dict['attributes']['alarming_policy'] = vnf_dict['id']
+                    vnf_dict['attributes'].update(alarm_url)
                     break
 
     def config_vnf(self, context, vnf_dict):
@@ -343,7 +318,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
         vnf_id = vnf_dict['id']
         LOG.debug(_('vnf_dict %s'), vnf_dict)
         self.mgmt_create_pre(context, vnf_dict)
-        self.add_alarm_url_to_vnf(vnf_dict)
+        self.add_alarm_url_to_vnf(context, vnf_dict)
         try:
             instance_id = self._vnf_manager.invoke(
                 driver_name, 'create', plugin=self,
@@ -364,6 +339,17 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
         name = vnf_info['name']
         if self._get_by_name(context, vnfm_db.VNF, name):
             raise exceptions.DuplicateResourceName(resource='VNF', name=name)
+
+        # if vnfd_template specified, create vnfd from template
+        # create template dictionary structure same as needed in create_vnfd()
+        if vnf_info.get('vnfd_template'):
+            vnfd_name = utils.generate_resource_name(name, 'inline')
+            vnfd = {'vnfd': {'attributes': {'vnfd': vnf_info['vnfd_template']},
+                             'name': vnfd_name,
+                             'template_source': 'inline',
+                             'service_types': [{'service_type': 'vnfd'}]}}
+            vnf_info['vnfd_id'] = self.create_vnfd(context, vnfd).get('id')
+
         vnf_attributes = vnf_info['attributes']
         if vnf_attributes.get('param_values'):
             param = vnf_attributes['param_values']
@@ -494,8 +480,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                 LOG.exception(_('_delete_vnf_wait'))
 
         self.mgmt_delete_post(context, vnf_dict)
-        vnf_id = vnf_dict['id']
-        self._delete_vnf_post(context, vnf_id, e)
+        self._delete_vnf_post(context, vnf_dict, e)
 
     def delete_vnf(self, context, vnf_id):
         vnf_dict = self._delete_vnf_pre(context, vnf_id)
@@ -527,7 +512,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                 vnf_dict['status'] = constants.ERROR
                 vnf_dict['error_reason'] = six.text_type(e)
                 self.mgmt_delete_post(context, vnf_dict)
-                self._delete_vnf_post(context, vnf_id, e)
+                self._delete_vnf_post(context, vnf_dict, e)
 
         self.spawn_n(self._delete_vnf_wait, context, vnf_dict, vim_auth,
                      driver_name)
@@ -633,7 +618,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                 _handle_vnf_scaling_post(constants.ACTIVE, mgmt_url)
                 # TODO(kanagaraj-manickam): Add support for config and mgmt
             except Exception as e:
-                LOG.error(_("Policy %s action is failed to complete") %
+                LOG.error(_("Policy %s action is failed to complete"),
                           policy['id'])
                 with excutils.save_and_reraise_exception():
                     self.set_vnf_error_status_reason(
@@ -673,24 +658,25 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
         vnfd_tmpl = yaml.load(vnf['vnfd']['attributes']['vnfd'])
         policy_list = []
 
-        if vnfd_tmpl.get('tosca_definitions_version'):
-            polices = vnfd_tmpl['topology_template'].get('policies', [])
-            for policy_dict in polices:
-                for name, policy in policy_dict.items():
-                    def _add(policy):
-                        p = self._make_policy_dict(vnf, name, policy)
-                        p['name'] = name
-                        policy_list.append(p)
+        polices = vnfd_tmpl['topology_template'].get('policies', [])
+        for policy_dict in polices:
+            for name, policy in policy_dict.items():
+                def _add(policy):
+                    p = self._make_policy_dict(vnf, name, policy)
+                    p['name'] = name
+                    policy_list.append(p)
 
-                    # Check for filters
-                    if filters.get('name'):
-                        if name == filters.get('name'):
-                            _add(policy)
-                            break
-                        else:
-                            continue
+                # Check for filters
+                if filters.get('name') or filters.get('type'):
+                    if name == filters.get('name'):
+                        _add(policy)
+                    if policy['type'] == filters.get('type'):
+                        _add(policy)
+                        break
+                    else:
+                        continue
 
-                    _add(policy)
+                _add(policy)
 
         return policy_list
 
@@ -714,38 +700,70 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
 
         return scale['scale']
 
-    def _validate_alarming_policy(self, context, policy):
-        vnf_id = policy['vnf']['id']
-        # validate policy type
-        type = policy['type']
-        if type not in constants.POLICY_ALARMING:
-            raise exceptions.VnfPolicyTypeInvalid(
-                type=type,
-                valid_types=constants.POLICY_ALARMING,
-                policy=policy['id']
-            )
+    def get_vnf_policy_by_type(self, context, vnf_id, policy_type=None, fields=None):             # noqa
+        policies = self.get_vnf_policies(context,
+                                         vnf_id,
+                                         filters={'type': policy_type})
+        if policies:
+            return policies[0]
+
+        raise exceptions.VnfPolicyTypeInvalid(type=constants.POLICY_ALARMING,
+                                              vnf_id=vnf_id)
+
+    def _validate_alarming_policy(self, context, vnf_id, trigger):
         # validate alarm status
-        if not self._vnf_alarm_monitor.process_alarm_for_vnf(policy):
+        if not self._vnf_alarm_monitor.process_alarm_for_vnf(vnf_id, trigger):
             raise exceptions.AlarmUrlInvalid(vnf_id=vnf_id)
 
-        # validate policy action
-        action = policy['action_name']
         policy_ = None
-        if action not in constants.DEFAULT_ALARM_ACTIONS:
-            policy_ = self.get_vnf_policy(context, action, vnf_id)
-            if not policy_:
-                raise exceptions.VnfPolicyNotFound(
-                    vnf_id=action,
-                    policy=policy['id']
-                )
-        LOG.debug(_("Policy %s is validated successfully") % policy)
-        return policy_
+        action_ = None
+        # validate policy action. if action is composite, split it.
+        # ex: SP1-in, SP1-out
+        action = trigger['action_name']
+        sp_action = action.split('-')
+        if len(sp_action) == 2:
+            bk_policy_name = sp_action[0]
+            bk_policy_action = sp_action[1]
+            policies_ = self.get_vnf_policies(context, vnf_id,
+                                              filters={'name': bk_policy_name})
+            if policies_:
+                policy_ = policies_[0]
+                action_ = bk_policy_action
+
+        if not policy_:
+            if action not in constants.DEFAULT_ALARM_ACTIONS:
+                policy_ = self.get_vnf_policy(context, action, vnf_id)
+        LOG.debug(_("Trigger %s is validated successfully"), trigger)
+        return policy_, action_
         # validate url
 
-    def _handle_vnf_monitoring(self, context, policy):
-        vnf_dict = policy['vnf']
-        if policy['action_name'] in constants.DEFAULT_ALARM_ACTIONS:
-            action = policy['action_name']
+    def _get_vnf_triggers(self, context, vnf_id, filters=None, fields=None):
+        policy = self.get_vnf_policy_by_type(
+            context, vnf_id, policy_type=constants.POLICY_ALARMING)
+        triggers = policy['properties']
+        vnf_trigger = dict()
+        for trigger_name, trigger_dict in triggers.items():
+            if trigger_name == filters.get('name'):
+                vnf_trigger['trigger'] = {trigger_name: trigger_dict}
+                vnf_trigger['vnf'] = policy['vnf']
+                break
+
+        return vnf_trigger
+
+    def get_vnf_trigger(self, context, vnf_id, trigger_name):
+        trigger = self._get_vnf_triggers(
+            context, vnf_id, filters={'name': trigger_name})
+        if not trigger:
+            raise exceptions.TriggerNotFound(
+                trigger_name=trigger_name,
+                vnf_id=vnf_id
+            )
+        return trigger
+
+    def _handle_vnf_monitoring(self, context, trigger):
+        vnf_dict = trigger['vnf']
+        if trigger['action_name'] in constants.DEFAULT_ALARM_ACTIONS:
+            action = trigger['action_name']
             LOG.debug(_('vnf for monitoring: %s'), vnf_dict)
             infra_driver, vim_auth = self._get_infra_driver(context, vnf_dict)
             action_cls = monitor.ActionPolicy.get_policy(action,
@@ -753,40 +771,40 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
             if action_cls:
                 action_cls.execute_action(self, vnf_dict)
 
-        if policy.get('bckend_policy'):
-            bckend_policy = policy['bckend_policy']
+        if trigger.get('bckend_policy'):
+            bckend_policy = trigger['bckend_policy']
             bckend_policy_type = bckend_policy['type']
-            cp = policy['properties']['resize_compute']['condition'].\
-                get('comparison_operator')
             if bckend_policy_type == constants.POLICY_SCALING:
+                if vnf_dict['status'] != constants.ACTIVE:
+                    LOG.info(_("Scaling Policy action skipped due to status:"
+                             " %(status)s for vnf: %(vnfid)s"),
+                             {"status": vnf_dict['status'],
+                              "vnfid": vnf_dict['id']})
+                    return
                 action = 'scaling'
                 scale = {}
                 scale.setdefault('scale', {})
-                scale['scale']['type'] = 'out' if cp == 'gt' else 'in'
+                scale['scale']['type'] = trigger['bckend_action']
                 scale['scale']['policy'] = bckend_policy['name']
+                infra_driver, vim_auth = self._get_infra_driver(context,
+                                                                vnf_dict)
                 action_cls = monitor.ActionPolicy.get_policy(action,
-                                                             vnf_dict)
+                                                             infra_driver)
                 if action_cls:
                     action_cls.execute_action(self, vnf_dict, scale)
 
     def create_vnf_trigger(
             self, context, vnf_id, trigger):
-        # Verified API: pending
-        # Need to use: _make_policy_dict, get_vnf_policies, get_vnf_policy
-        # action: scaling, refer to template to find specific scaling policy
-        # we can extend in future to support other policies
-        # Monitoring policy should be describe in heat_template_yaml.
-        # Create first
-        policy_ = self.get_vnf_policy(context,
-                                      trigger['trigger']['policy_name'],
-                                      vnf_id)
-        policy_.update({'action_name': trigger['trigger']['action_name']})
-        policy_.update({'params': trigger['trigger']['params']})
-        bk_policy = self._validate_alarming_policy(context, policy_)
+        trigger_ = self.get_vnf_trigger(
+            context, vnf_id, trigger['trigger']['policy_name'])
+        trigger_.update({'action_name': trigger['trigger']['action_name']})
+        trigger_.update({'params': trigger['trigger']['params']})
+        bk_policy, bk_action = self._validate_alarming_policy(
+            context, vnf_id, trigger_)
         if bk_policy:
-            policy_.update({'bckend_policy': bk_policy})
-        self._handle_vnf_monitoring(context, policy_)
-
+            trigger_.update({'bckend_policy': bk_policy,
+                             'bckend_action': bk_action})
+        self._handle_vnf_monitoring(context, trigger_)
         return trigger['trigger']
 
     def get_vnf_resources(self, context, vnf_id, fields=None, filters=None):
